@@ -1,36 +1,28 @@
 from __future__ import absolute_import
 
 from ocrd_utils import getLogger
-from ocrd import Processor
+from ocrd_validators import ParameterValidator
 
 from .config import OCRD_TOOL
 from .recognize import TesserocrRecognize
 
 TOOL = 'ocrd-tesserocr-segment-line'
+BASE_TOOL = 'ocrd-tesserocr-recognize'
 
-class TesserocrSegmentLine(Processor):
-
+class TesserocrSegmentLine(TesserocrRecognize):
     def __init__(self, *args, **kwargs):
-        kwargs['ocrd_tool'] = OCRD_TOOL['tools'][TOOL]
-        kwargs['version'] = OCRD_TOOL['version']
+        kwargs.setdefault('ocrd_tool', OCRD_TOOL['tools'][TOOL])
         super().__init__(*args, **kwargs)
+        if hasattr(self, 'parameter'):
+            self.parameter['overwrite_segments'] = self.parameter['overwrite_lines']
+            del self.parameter['overwrite_lines']
+            self.parameter['segmentation_level'] = "line"
+            self.parameter['textequiv_level'] = "line"
+            # add default params
+            assert ParameterValidator(OCRD_TOOL['tools'][BASE_TOOL]).validate(self.parameter).is_valid
+            self.logger = getLogger('processor.TesserocrSegmentLine')
 
-        if hasattr(self, 'workspace'):
-            recognize_kwargs = {**kwargs}
-            recognize_kwargs.pop('dump_json', None)
-            recognize_kwargs.pop('dump_module_dir', None)
-            recognize_kwargs.pop('show_help', None)
-            recognize_kwargs.pop('show_version', None)
-            recognize_kwargs['parameter'] = self.parameter
-            recognize_kwargs['parameter']['overwrite_segments'] = self.parameter['overwrite_lines']
-            del recognize_kwargs['parameter']['overwrite_lines']
-            recognize_kwargs['parameter']['segmentation_level'] = "line"
-            recognize_kwargs['parameter']['textequiv_level'] = "line"
-            self.recognizer = TesserocrRecognize(self.workspace, **recognize_kwargs)
-            self.recognizer.logger = getLogger('processor.TesserocrSegmentLine')
-
-    def process(self):
-        """Performs (text) line segmentation with Tesseract on the workspace.
+TesserocrSegmentLine.process.__doc__ = """Performs (text) line segmentation with Tesseract on the workspace.
         
         Open and deserialize PAGE input files and their respective images,
         then iterate over the element hierarchy down to the (text) region level,
@@ -48,4 +40,3 @@ class TesserocrSegmentLine(Processor):
         
         Produce a new output file by serialising the resulting hierarchy.
         """
-        return self.recognizer.process()
